Package: quadrupen
Type: Package
Title: Sparse and Group Sparse Linear Models
Version: 1.0-0
Date: 2026-06-05
Authors@R: 
        person("Julien", "Chiquet", email = "julien.chiquet@inrae.fr", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-3629-3429"))
Description: Fits the solution paths of classical sparse regression models with efficient active set algorithms by solving small sub-problems. Include LASSO, SCAD, MCP, (Sparse) Group-LASSO, Cooperative-LASSO, (Group) LAVA, (Generalized) Fused-Lasso and (Generalized) Elastic-Net. Also provides methods for model selection purpose (information criteria, cross-validation, stability selection).
URL: https://github.com/jchiquet/quadrupen,
        https://jchiquet.github.io/quadrupen/
BugReports: https://github.com/jchiquet/quadrupen/issues
License: GPL (>= 3)
Encoding: UTF-8
Language: en-US
SystemRequirements: GNU make
Depends: R (>= 4.1.0)
Imports: dplyr, ggplot2, grid, Matrix, methods, parallel, R6, Rcpp,
        rlang, scales, tidyr, lifecycle
Suggests: elasticnet, glmnet, igraph, knitr, lars, MASS, ncvreg,
        grpreg, rmarkdown, testthat
LinkingTo: Rcpp, RcppArmadillo
Collate: 'init.R' 'quadrupen-package.R' 'QuadrupenFit-R6Class.R'
        'quadrupen-S3Methods.R' 'Regularizers-R6Class.R'
        'DataModel-R6Class.R' 'StabilityPath-R6Class.R'
        'CrossValidation-R6Class.R' 'InformationCriteria-R6Class.R'
        'bounded-reg.R' 'fused-lasso.R' 'sparse_lm.R'
        'sparse_lm_aliases.R' 'group_sparse_lm.R'
        'group_sparse_lm_aliases.R' 'group-lava.R' 'lava.R' 'utils.R'
        'ridge.R' 'RcppExports.R'
Packaged: 2026-06-05 13:35:07 UTC; jchiquet
Config/roxygen2/version: 8.0.0
VignetteBuilder: knitr
NeedsCompilation: yes
Author: Julien Chiquet [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3629-3429>)
Maintainer: Julien Chiquet <julien.chiquet@inrae.fr>
Repository: CRAN
Date/Publication: 2026-06-05 14:40:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-06-05 18:49:26 UTC; unix
Archs: quadrupen.so.dSYM
