# Generated by roxygen2: do not edit by hand

S3method(coef,QuadrupenFit)
S3method(criteria,QuadrupenFit)
S3method(cross_validate,QuadrupenFit)
S3method(deviance,QuadrupenFit)
S3method(fitted,QuadrupenFit)
S3method(plot,CrossValidation)
S3method(plot,QuadrupenFit)
S3method(plot,StabilityPath)
S3method(predict,QuadrupenFit)
S3method(residuals,QuadrupenFit)
S3method(selection,StabilityPath)
S3method(stability,QuadrupenFit)
export(BoundedRegressionFit)
export(CrossValidation)
export(DataModel)
export(FusedLassoFit)
export(GroupLavaFit)
export(InformationCriteria)
export(LavaFit)
export(QuadrupenFit)
export(RidgeRegressionFit)
export(SparseFit)
export(SparseGroupFit)
export(StabilityPath)
export(bounded.reg)
export(bounded_reg)
export(coop_lasso)
export(criteria)
export(cross_validate)
export(elastic.net)
export(elastic_net)
export(fused_lasso)
export(group_l1linf)
export(group_lasso)
export(group_lava)
export(group_sparse_lm)
export(lasso)
export(lava)
export(mcp)
export(ridge)
export(scad)
export(selection)
export(sparse_coop_lasso)
export(sparse_group_l1linf)
export(sparse_group_lasso)
export(sparse_lm)
export(stability)
import(Matrix)
import(R6)
import(dplyr)
import(ggplot2)
import(grid)
import(methods)
import(parallel)
import(scales)
importFrom(Rcpp,sourceCpp)
importFrom(dplyr,filter)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(graphics,plot)
importFrom(lifecycle,badge)
importFrom(lifecycle,deprecate_warn)
importFrom(methods,as)
importFrom(rlang,.data)
importFrom(stats,deviance)
importFrom(stats,fitted)
importFrom(stats,predict)
importFrom(stats,residuals)
importFrom(stats,setNames)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,starts_with)
useDynLib(quadrupen, .registration = TRUE)
